package Padre::Plugin::Moose::CanGenerateCode;

use Moose::Role;
use namespace::clean;

our $VERSION = '0.07';

requires 'generate_code';

1;

__END__

=pod

=head1 NAME

Padre::Plugin::Moose::CanGenerateCode - Something that can generate code

=cut
